CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: error.fpp,v 1.4 2014/03/14 13:57:06 fclement Exp $
      
C     
C     Error messages.
C     
      
#include "pio_constants.h"

C     
C     Errors related to external aggressions.
C     
      
      subroutine error_read_error(sbr, ic)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer ic
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'read error on input channel ', ic, '.'
      
      call exit(PIO_READ_ERROR)
      
      return
      end

      
      subroutine error_write_error(sbr, oc)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer oc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'write error on output channel ', oc, '.'
      
      call exit(PIO_WRITE_ERROR)
      
      return
      end
            
C     
C     Errors related to the Fortran implementation of the PIO library.
C     
      
      subroutine error_buffer_overflow(sbr, io)
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     buffer_name(io), ' buffer overflow.'
      
      call exit(PIO_BUFFER_OVERFLOW_ERROR)
      
      return
      end
      
      
      subroutine error_feature_not_yet_implemented(sbr, ftr)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), ftr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'the feature ', ftr, ' is not yet implemented.'
      
      call exit(PIO_NOT_YET_IMPLEMENTED_ERROR)
      
      return
      end
      
      
      subroutine error_float_buffer_overflow(sbr)
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     buffer_name(PIO_IN), ' float buffer overflow.'
      
      call exit(PIO_BUFFER_OVERFLOW_ERROR)
      
      return
      end
      
      
      subroutine error_incoherent_string_length(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'incoherent string length in the memory mapping, ',
     +     'first read ', good, ', then read ', bad, '.'
      
      call exit(PIO_INCOHERENT_STRING_LENGTH_ERROR)
      
      return
      end
      
      
      subroutine error_ident_buffer_overflow(sbr)
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     buffer_name(PIO_IN), ' ident buffer overflow.'
      
      call exit(PIO_BUFFER_OVERFLOW_ERROR)
      
      return
      end
      
      
      subroutine error_ident_too_long(sbr, n)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'ident has length ', n, ' which is longer than the ',
     +     'maximum ', PIO_MAX_IDENT_LENGTH, '.'
      
      call exit(PIO_IDENT_TOO_LONG_ERROR)
      
      return
      end
      
      
      subroutine error_integer_buffer_overflow(sbr)
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     buffer_name(PIO_OUT), ' integer buffer overflow.'
      
      call exit(PIO_BUFFER_OVERFLOW_ERROR)
      
      return
      end
      
      
      subroutine error_internal_error(sbr)
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'internal error.'
      
      call exit(PIO_INTERNAL_ERROR)
      
      return
      end
      
       
      subroutine error_invalid_data_access(sbr, io, inv, ptr, good, bad)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*), inv*(*), ptr*(*)
      integer io, good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     memory_name(io), ' communication ', inv,
     +     ' invariant violation.'
      write(PIO_STDERR, *) 'Expecting for a ', ptr,
     +     ' in location at most ', good, ' but found it in location ',
     +     bad, '.'
      
      call exit(PIO_INVARIANT_ERROR)
      
      return
      end
      
      
      subroutine error_invalid_memory_mapping_access(sbr, io, loc)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'accessing slot ', loc, ' would access out of the memory ',
     +     'mapping for ', memory_name(io), ' communications.'
      
      call exit(PIO_INVALID_MEMORY_MAPPING_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_length_too_long(sbr, len, lim)
       
      implicit none
     
C     Input arguments.
      character sbr*(*)
      integer len, lim
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'specified length ', len, ' ',
     +     'is greater than length limit ', lim, '.'
      
      call exit(PIO_LENGTH_TOO_LONG_ERROR)
      
      return
      end
      
       
      subroutine error_memory_access(sbr, io, loc)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'access to (', loc, ') in the memory for ', memory_name(io),
     +     ' communications is out of bounds.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_memory_mapping_overflow(sbr, io)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'overflow in the memory mapping for ', memory_name(io),
     +     ' communications.'
      
      call exit(PIO_MEMORY_MAPPING_OVERFLOW_ERROR)
      
      return
      end
      
     
      subroutine error_out_of_bounds(sbr, io, ptr)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*), ptr*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     memory_name(io), ' communication ', ptr,
     +     ' is out of bounds.'
      
      call exit(PIO_INVARIANT_ERROR)
      
      return
      end
      
      
      subroutine error_out_of_value_memory_access(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'attempt to read up to location ', bad, ' which is further',
     +     ' the limit of the current value ', good, '.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_overflow(sbr, io, buf)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*), buf*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     memory_name(io), ' communication ', buf, ' overflow.'
      
      call exit(PIO_INVARIANT_ERROR)
      
      return
      end
      
      
      subroutine error_rand_fetch_memory_access(sbr, io, loc, end_loc)
C     Attempt to read out of the memory.
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'attempt to read from location (', loc, ') ',
     +     'to location (', end_loc, ') would read out of the memory ',
     +     'for ', memory_name(io), ' communications.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
     
      subroutine error_rand_fetch_range_memory_access
     +     (sbr, io, loc, end_loc)
C     Attempt to read a ''negative'' range.
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'attempt to read from the higher location (', loc, ') ',
     +     'to the lower location (', end_loc, ') ',
     +     'in the memory for ', memory_name(io), ' communications.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_rand_store_memory_access(sbr, io, loc, end_loc)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'attempt to write from location (', loc, ') ',
     +     'to location (', end_loc, ') would write out of the memory ',
     +     'for ', memory_name(io), ' communications.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_rand_store_range_memory_access
     +     (sbr, io, loc, end_loc)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'attempt to write from the higher location (', loc, ') ',
     +     'to the lower location (', end_loc, ') ',
     +     'in the memory for ', memory_name(io), ' communications.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_routine_not_yet_implemented(sbr)
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'not yet implemented.'
      
      call exit(PIO_NOT_YET_IMPLEMENTED_ERROR)
      
      return
      end
      
      
      subroutine error_seq_fetch_memory_access(sbr, io, loc, end_loc)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'reading from (', loc, ') to (', end_loc, ') ',
     +     'would read unwritten slot(s) in the memory for ',
     +     memory_name(io), ' communications.'
      
      call exit(PIO_MEMORY_ACCESS_ERROR)
      
      return
      end
      
      
      subroutine error_string_buffer_overflow(sbr)
      
      implicit none
      
      include 'buffer_name_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     buffer_name(PIO_IN), ' string buffer overflow.'
      
      call exit(PIO_BUFFER_OVERFLOW_ERROR)
      
      return
      end
      
      
      subroutine error_too_long_string_value(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'fetching for a string of length at most ', good,
     +     ' but found a string of length ', bad, '.'
      
      call exit(PIO_TOO_LONG_STRING_VALUE_ERROR)
      
      return
      end
      
      
      subroutine error_too_many_array_dimensions(sbr, p)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer p
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'dimension ', p, ' is greater than the maximum ',
     +     MAX_ARRAY_DIMENSION, '.'
      
      call exit(PIO_TOO_MANY_ARRAY_DIMENSIONS_ERROR)
      
      return
      end
      
      
      subroutine error_too_many_values(sbr, n)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'number of values ', n, ' is greater than the maximum ',
     +     MAX_VALUE_NUMBER, '.'
      
      call exit(PIO_TOO_MANY_VALUES_ERROR)
      
      return
      end
      
      
      subroutine error_too_short_list(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for a list of length at least ', good,
     +     ' but found one of length ', bad, '.'
      
      call exit(PIO_TOO_SHORT_LIST_ERROR)
      
      return
      end
      
      
      subroutine error_unbound_layout_kind(sbr, lkv)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer lkv
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'unbound layout kind ', lkv, '.'
      
      call exit(PIO_UNBOUND_LAYOUT_KIND_ERROR)
      
      return
      end
      
      
      subroutine error_unset_value_descriptor_address(sbr, io, i)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'address of value number ', i, ' in the memory mapping for ',
     +     memory_name(io), ' communications was unset when accessing ',
     +     'location relative to it.'
      
      call exit(PIO_UNSET_VALUE_DESCRIPTOR_ADDRESS_ERROR)
      
      return
      end
      
      
      subroutine error_unwritten_data_access
     +     (sbr, io, inv, ptr, good, bad)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*), inv*(*), ptr*(*)
      integer io, good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     memory_name(io), ' communication ', inv,
     +     ' invariant violation.'
      write(PIO_STDERR, *) 'Expecting for a ', ptr,
     +     ' in location at most ', good, ' but found it in location ',
     +     bad, '.'
      
      call exit(PIO_INVARIANT_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_communication_type(sbr, typ)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer typ
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'wrong communication type ''', typ, '''.'
      
      call exit(PIO_WRONG_COMMUNICATION_TYPE_ERROR)
      
      return
      end
      
      
      subroutine error_too_large_size(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'size ', bad, ' is greater than the maximum ', good, '.'
      
      call exit(PIO_TOO_LARGE_SIZE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_size(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for size ''', good, ''', ',
     +     'but found size ''', bad, '''.'
      
      call exit(PIO_WRONG_SIZE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_dimension(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for dimension ''', good, ''', ',
     +     'but found dimension ''', bad, '''.'
      
      call exit(PIO_WRONG_DIMENSION_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_float_compound_size(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for a float compound of size ', good * FLOAT_SIZE,
     +     ' bytes but found a float compound of size ', bad, ' bytes.'
      
      call exit(PIO_WRONG_FLOAT_COMPOUND_SIZE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_int_compound_size(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for an integer compound of size ', good * INT_SIZE,
     +     ' bytes but found an integer compound of size ',
     +     bad, ' bytes.'
      
      call exit(PIO_WRONG_INT_COMPOUND_SIZE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_memory_clean(sbr, io, good, bad)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      logical good, bad
      character sbr*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'the memory for ', memory_name(io), ' communication ',
     +     'is not in the expected clean state.'
      write(PIO_STDERR, *) 'Waiting for a ',
     +     good, ' memory clean state ', 'but found a ',
     +     bad, ' memory clean state.'
      
      call exit(PIO_WRONG_MEMORY_CLEAN_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_memory_mapping_clean(sbr, io, good, bad)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      logical good, bad
      character sbr*(*)
      integer io
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'the memory mapping for ', memory_name(io), ' communication',
     +     ' is not in the expected clean state.'
      write(PIO_STDERR, *) 'Waiting for a ', good, ' memory mapping ',
     +     'clean state but found a ', bad, ' memory mapping ',
     +     'clean state.'
      
      call exit(PIO_WRONG_MEMORY_MAPPING_CLEAN_ERROR)
      
      return
      end
      
      
      subroutine pio_error_wrong_number_of_values(sbr, io, good, bad)
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for ', good, ' value(s), ',
     +     'but found ', bad, ' value(s) written in the memory for ',
     +     memory_name(io), ' communication.'
      
      call exit(PIO_WRONG_NUMBER_OF_VALUES_ERROR)
      
      return
      end
      
C     
C     Errors related either to the Fortran implementation of the PIO
C     library or to the contents of the received communication.
C     
      
      subroutine error_assertion_failure(sbr, msg)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), msg*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'assertion failure, ', msg
      
      call exit(PIO_ASSERTION_FAILURE_ERROR)
      
      return
      end
      
C     
C     Errors related to the contents of the received communication.
C     
      
      subroutine error_char_not_found(sbr, c)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'character ''', c, ''' not found.'
      
      call exit(PIO_CHAR_NOT_FOUND_ERROR)
      
      return
      end
      
      
      subroutine error_char_syntax(sbr, good, bad)
C     Waiting for character good, but found character bad.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for character ''', good, ''', ',
     +     'but found character ''', bad, '''.'
      
      call exit(PIO_CHAR_SYNTAX_ERROR)
      
      return
      end
      
      
      subroutine error_end_of_file(sbr, ic)
C     Reach end-of-file while reading on input channel ic.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer ic
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'reached end-of-file while reading on input channel ', ic,
     +     '.'
      
      call exit(PIO_END_OF_FILE_ERROR)
      
      return
      end
      
      
      subroutine error_heterogeneous_tuple_type(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for a tuple item of type ', good,
     +     'but found an item of type ', bad, '.'
      
      call exit(PIO_HETEROGENEOUS_TUPLE_TYPE_ERROR)
      
      return
      end
      
      
      subroutine error_incoherent_array_markers(sbr, p, q)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer p, q
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'incoherent array markers, first read ', p, ', ',
     +     'then read ', q, '.'
      
      call exit(PIO_INCOHERENT_ARRAY_MARKERS_ERROR)
      
      return
      end
      
      
      subroutine error_length_too_short(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for ''', good, ''' number of items, ',
     +     'but found ''', bad, ''' items.'
      
      call exit(PIO_LENGTH_TOO_SHORT_ERROR)
      
      return
      end
      
      
      subroutine error_malformed_escaped_char(sbr, c)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'malformed escaped character, waiting for a decimal digit, ',
     +     'but found character ''', c, '''.'
      
      call exit(PIO_MALFORMED_ESCAPED_CHAR_ERROR)
      
      return
      end
      
      
      subroutine error_non_empty_line(sbr, bad)
C     Waiting for an empty line, but found a line starting by string bad.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), bad*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for an empty line, ',
     +     'but found a line starting by ''', bad, '''.'
      
      call exit(PIO_LINE_SYNTAX_ERROR)
      
      return
      end
      
      
      subroutine error_unexpected_char(sbr, bad)
       
      implicit none
     
C     Input arguments.
      character sbr*(*), bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'found unexpected character ''', bad, '''.'
      
      call exit(PIO_CHAR_SYNTAX_ERROR)
      
      return
      end
      
      
      subroutine error_unknown_service_name(sbr, len, sname)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), sname*(*)
      integer len
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'unknown service name ', sname(1:len), '.'
      
      call exit(PIO_UNKNOWN_SERVICE_NAME_ERROR)
      
      return
      end
      
      
      subroutine error_unbound_char_in_typed_value_type(sbr, c)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c(1)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'unbound char ''', c, ''' in typed value type.'
      
      call exit(PIO_UNBOUND_CHAR_IN_TYPED_VALUE_TYPE_ERROR)
      
      return
      end
      
      
      subroutine error_unknown_fortran_typed_value_type(sbr, typ)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer typ
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'unknown Fortran typed value type ''', typ, '''.'
      
      call exit(PIO_UNKNOWN_FORTRAN_TYPED_VALUE_TYPE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_communication_format(sbr, c)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'wrong communication format ''', c, '''.'
      
      call exit(PIO_WRONG_COMMUNICATION_FORMAT_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_current_line(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for line |', good, '| but found line |', bad, '|.'
      
      call exit(PIO_LINE_SYNTAX_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_float_size(sbr, bad)
C     Waiting for an float of size FLOAT_SIZE, but found one of size bad.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for an float of size ', FLOAT_SIZE,
     +     ' but found an float of size ', bad, '.'
      
      call exit(PIO_WRONG_FLOAT_SIZE_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_string_length(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for a string of length ', good,
     +     ' but found a string of length ', bad, '.'
      
      call exit(PIO_WRONG_STRING_LENGTH_ERROR)
      
      return
      end
      
      
      subroutine error_wrong_typed_value_type(sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer good, bad
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'waiting for a value of type ', good,
     +     ' but found a value of type ', bad, '.'
      
      call exit(PIO_WRONG_TYPED_VALUE_TYPE_ERROR)
      
      return
      end
      
      
      subroutine error_empty_token(sbr)
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
      write(PIO_STDERR, *) '[Pio/Fortran] Fatal error in ', sbr, ': ',
     +     'empty token.'
      
      call exit(PIO_EMPTY_TOKEN_ERROR)
      
      return
      end
